#-------------------------------------------------------------------------------
# Name:        Least Cost Transect Analysis LCTA - LCP SLAVE
# Purpose:      Version 1.2
#
# Author:      Maarten J. van Strien
#
# Created:     02.05.2012
# Copyright:  
# Licence:
#-------------------------------------------------------------------------------

# importing functions
from dbfpy import dbf
import time
import os
import arcgisscripting
import sys

# Define all the variables necessary for this SLAVE program.
wrkspc3 = sys.argv[1]
le_table3 = sys.argv[2]
log_name3 = sys.argv[3]
prs_table3 = sys.argv[4]
samp_locs3 = sys.argv[5]
sampID3 = sys.argv[6]
le3 = sys.argv[7]
buff3 = int(sys.argv[8])
fctr3 = int(sys.argv[9])
cllsz3 = int(sys.argv[10])

# Setting the initial environment
gp = arcgisscripting.create()
gp.Workspace = wrkspc3
gp.Cellsize = cllsz3
gp.SetProduct('ArcInfo')
gp.CheckOutExtension("Spatial")
gp.Overwriteoutput = 1
gp.Extent = "extent"

# Read the table with all the landscape elements (le_table)
le_df = dbf.Dbf(wrkspc3+"\\"+le_table3)

# Read the table with all the population pairs (prs_table)
pairs = dbf.Dbf(wrkspc3+"\\"+prs_table3)

# Make a layer out of the samp_locs point shapefile for quick selections
gp.MakeFeatureLayer(samp_locs3,"samp_lyr")

# Printing loop information
log = open(wrkspc3+"\\"+log_name3, 'a')
log.write("\n")
log.write("## Starting LCTA calculation for: mh = "+le3+", sf = "+fctr3.__str__()+", buff = "+buff3.__str__()+"\n")
log.close()

# Creates a tmp_tab
gp.CreateTable_management(wrkspc3, "temp_tab.dbf", "out_tab.dbf")

# Create a list with all the cost_distance and backlink rasters that have been created in this analysis
cd_list = []
bl_list = []

# Start of the loop to calculate least cost paths between all start and end nodes
for pair in pairs:
    # Starting the time measurement for this loop
    start = time.time()

    # Setting start and end populations
    start_node = pair.__getitem__("NODE1")
    end_node = pair.__getitem__("NODE2")

    # Check in the population codes are numeric (second) or character  (first)
    try:
        i = float(start_node)
    except ValueError, TypeError:
        sn_where = "\"y\" = 'z'"
        en_where = "\"y\" = 'z'"
    else:
        sn_where = "\"y\" = z"
        en_where = "\"y\" = z"

    # Set the name of the LCP in this loop
    lcp_name = "sp_"+start_node.__str__()+"_"+end_node.__str__()+"_f_"+fctr3.__str__()

    # Set the name for the cost_distance and backlink rasters in this loop
    cd_name = "cd"+start_node.__str__()+le3[0:5]+fctr3.__str__()
    bl_name = "bl"+start_node.__str__()+le3[0:5]+fctr3.__str__()

    # Check whether the LCP has already been calculated and if not, if the cost distance has already been calculated for this start_node
    if gp.Exists("sh_pth_"+le3+"/"+lcp_name+".shp") == 1:
        gp.Copy_management("sh_pth_"+le3+"/"+lcp_name+".shp", lcp_name+".shp")
    elif gp.Exists(cd_name) == 0:
        sn_where = sn_where.replace('y',sampID3)
        sn_where = sn_where.replace('z',start_node.__str__())
        gp.SelectLayerByAttribute_management("samp_lyr", "NEW_SELECTION", sn_where)
        gp.CostDistance_sa("samp_lyr", le3+"_f_"+fctr3.__str__(), cd_name, "", bl_name)

        # Add a line to the log file to indicate that the cost_distance raster has been created.
        log = open(wrkspc3+"\\"+log_name3, 'a')
        log.write("#### Created cost distance raster: "+cd_name+"; and backlink raster: "+bl_name+" \n")
        log.close()

        # Create a list of start_nodes so the costdistance and backlink rasters can be deleted
        cd_list.append(cd_name)
        bl_list.append(bl_name)

    # Select the end node in the layer of sampling points.
    en_where = en_where.replace('y',sampID3)
    en_where = en_where.replace('z',end_node.__str__())
    gp.SelectLayerByAttribute_management("samp_lyr", "NEW_SELECTION", en_where)

    # Calculate the cost-path between start and end node
    if gp.Exists(lcp_name+".shp") == 0:
        gp.CostPath_sa("samp_lyr", cd_name, bl_name, "cost_path", "BEST_SINGLE", "")

        # Export the cost of the LCP
        vat = gp.searchcursor("cost_path")
        row_n = vat.next()
        cost1_list = []
        while row_n:
            cost1_list.append(row_n.pathcost)
            row_n = vat.next()
        cost1 = sum(cost1_list)
        del vat, row_n, cost1_list

        # Turns the raster LCP to a polyline
        gp.RasterToPolyline_conversion("cost_path","temp.shp", "ZERO", "", "SIMPLIFY")
        gp.Dissolve_management("temp.shp", lcp_name+".shp")
        gp.delete_management("temp.shp")
        gp.delete_management("cost_path")

        # Calculate the length of the LCP.
        gp.Addfield_management(lcp_name+".shp", "LENGTH", "DOUBLE")
        gp.CalculateField_management(lcp_name+".shp", "LENGTH", "float(!SHAPE.LENGTH!)", "PYTHON")
        gp.Addfield_management(lcp_name+".shp", "COST", "DOUBLE")
        gp.CalculateField_management(lcp_name+".shp", "COST", cost1, "PYTHON")

        # Copy the lcp shapefile to a folder with the name of the le in the working directory.
        gp.Copy_management(lcp_name+".shp", "sh_pth_"+le3+"/"+lcp_name+".shp")

        del cost1

    # Extract the length and the cost of the LCP shapefile
    lc_database = dbf.Dbf(wrkspc3+"\\"+lcp_name+".dbf")
    length = ("%.2f" % sum([row["LENGTH"] for row in lc_database]))
    cost2 = ("%.2f" % sum([row["COST"] for row in lc_database]))
    lc_database.close()

    # Buffers the LCP with a specific distance (buff).
    gp.Buffer_analysis(lcp_name+".shp", "buf_"+lcp_name+".shp", buff3, "FULL", "ROUND", "ALL")

    # Counts the number of pixels in the complete path.
    gp.ExtractByMask_sa("extent", "buf_"+lcp_name+".shp", "temp_rast")
    vat = gp.searchcursor("temp_rast")
    row_n = vat.next()
    all_pix_list = []
    while row_n:
        all_pix_list.append(row_n.count)
        row_n = vat.next()
    all_pix = sum(all_pix_list)
    del vat, row_n, all_pix_list
    gp.delete_management("temp_rast")

    # Fill the "temp_tab.dbf" for this loop
    temp_tab = dbf.Dbf(wrkspc3+"\\"+"temp_tab.dbf")
    temp_tab_row = temp_tab.newRecord()
    temp_tab_row["Field1"] = 0
    temp_tab_row["NODE1"] = start_node.__str__()
    temp_tab_row["NODE2"] = end_node.__str__()
    temp_tab_row["LENGTH"] = float(length)
    temp_tab_row["COST"] = float(cost2)
    temp_tab_row["ALL_PIX"] = float(all_pix)

    # Loop to add the proportion of each landscape element in the "temp_tab.dbf".
    for rec in le_df:
        le4 = rec.__getitem__("LANDEL")
        gp.ExtractByMask_sa(le4, "buf_"+lcp_name+".shp", "temp_rast")
        if gp.Exists("temp_rast") == 0:
            log = open(wrkspc3+"\\"+log_name3, 'a')
            log.write("!!!!!!!! ERROR: temp_rast not created\n")
            log.close()
        max_val = gp.GetRasterProperties_management("temp_rast", "MAXIMUM")
        if max_val > 0:
            vat = gp.searchcursor("temp_rast")
            row_n = vat.next()
            le_pix_list = []
            while row_n:
                le_pix_list.append(row_n.count)
                row_n = vat.next()
            le_pix = sum(le_pix_list)
            del vat, row_n, le_pix_list
        else:
            le_pix = 0
        temp_tab_row[le4] = float(le_pix)/float(all_pix)
        gp.delete_management("temp_rast")

    # Close the temp_tab.dbf
    temp_tab_row.store()
    temp_tab.close()

    # Delete the created variables
    del length, cost2, all_pix, le_pix, max_val, rec, cd_name, bl_name

    # Delete the LCP and transect shapefiles
    gp.delete_management(lcp_name+".shp")
    gp.delete_management("buf_"+lcp_name+".shp")

    # Print loop summary and elapsed time.
    end = time.time()
    log = open(wrkspc3+"\\"+log_name3, 'a')
    log.write("## Finished calculations between deme "+start_node.__str__()+" and "+end_node.__str__()+" in "+("%.1f" % (end - start))+" seconds \n")
    log.close()

# Delete all the created costdistance and backlink rasters
for cd in cd_list:
    gp.delete_management(cd)

for bl in bl_list:
    gp.delete_management(bl)

del cd_list, bl_list

# Rename the temp_tab.dbf
if gp.Exists("sp_"+le3+"_f_"+fctr3.__str__()+"_b_"+buff3.__str__()+".dbf") == 1:
    os.remove(os.path.join(wrkspc3, "sp_"+le3+"_f_"+fctr3.__str__()+"_b_"+buff3.__str__()+".dbf"))
os.rename(wrkspc3+"\\temp_tab.dbf", wrkspc3+"\\sp_"+le3+"_f_"+fctr3.__str__()+"_b_"+buff3.__str__()+".dbf")



