#-------------------------------------------------------------------------------
# Name:        Least Cost Transect Analysis LCTA - PREP SLAVE
# Purpose:      Version 1.2
#
# Author:      Maarten J. van Strien
#
# Created:     02.05.2012
# Copyright:   
# Licence:
#-------------------------------------------------------------------------------

# importing functions
from dbfpy import dbf
import time
import os
import arcgisscripting
import sys

# Define all the variables necessary for this SLAVE program.
wrkspc2 = sys.argv[1]
le_table2 = sys.argv[2]
log_name2 = sys.argv[3]
cllsz2 = int(sys.argv[4])
stdyr2 = sys.argv[5]
sf2 = int(sys.argv[6])
shpfl2 = sys.argv[7]
clmn2 = sys.argv[8]
le2 = sys.argv[9]

# Setting the initial environment
gp = arcgisscripting.create()
gp.Workspace = wrkspc2
gp.Cellsize = cllsz2
gp.SetProduct('ArcInfo')
gp.CheckOutExtension("Spatial")
gp.Overwriteoutput = 1

# Create the 'extent' raster if it does not exist yet
if gp.Exists("extent") == 0:
    gp.FeaturetoRaster_conversion(stdyr2, "Name", "extent", cllsz2)

# All subsequent calculations use 'extent' to determine their extent
gp.Extent = "extent"

# Create a 0-1 raster for all the landscape elements in le_table
if gp.Exists(le2) == 0:
    le_where = "\"y\" = 'z'"
    le_where = le_where.replace('z',le2)
    le_where = le_where.replace('y',clmn2)
    gp.MakeFeatureLayer(shpfl2,"lyr")
    gp.SelectLayerByAttribute_management("lyr", "NEW_SELECTION", le_where)
    gp.FeaturetoRaster_conversion("lyr", clmn2, le2, cllsz2)
    gp.delete_management("lyr")
    del le_where

# Create a folder in which all the shortest paths will be saved.
dirname = wrkspc2+"\\sh_pth_"+le2
if not os.path.isdir(dirname):
    os.mkdir(dirname)

# For each combination of SF and LE a resistance surface is created
rclss_ranges = "1 1;NoData "
f = 2**sf2
rclss_ranges = rclss_ranges.__add__(f.__str__())
gp.Reclassify_sa(le2, "Value", rclss_ranges, "rct_"+le2, "DATA")
gp.Times_sa("rct_"+le2, "extent", le2+"_f_"+sf2.__str__())
gp.delete_management("rct_"+le2)
del rclss_ranges, f

# Add a line to the log file
log = open(wrkspc2+"\\"+log_name2, 'a')
log.write("++ Finished executing the preporatory steps for "+le2+" and "+sf2.__str__()+" at "+time.strftime("%H:%M:%S", time.localtime())+"\n")
log.close()

# Delete most of the variables.
del sf2, shpfl2, clmn2, le2, dirname, wrkspc2, le_table2, log_name2, cllsz2, stdyr2
