#-------------------------------------------------------------------------------
# Name:        Least Cost Transect Analysis LCTA - STR. LINE SLAVE
# Purpose:      Version 1.2
#
# Author:      Maarten J. van Strien
#
# Created:     02.05.2012
# Copyright: 
# Licence:
#-------------------------------------------------------------------------------

# importing functions
from dbfpy import dbf
import time
import os
import arcgisscripting
import sys
import math

# Define all the variables necessary for this SLAVE program.
wrkspc4 = sys.argv[1]
le_table4 = sys.argv[2]
log_name4 = sys.argv[3]
prs_table4 = sys.argv[4]
samp_locs4 = sys.argv[5]
sampID4 = sys.argv[6]
buff4 = int(sys.argv[7])
cllsz4 = int(sys.argv[8])

# Setting the initial environment
gp = arcgisscripting.create()
gp.Workspace = wrkspc4
gp.Cellsize = cllsz4
gp.SetProduct('ArcInfo')
gp.CheckOutExtension("Spatial")
gp.Overwriteoutput = 1
gp.Extent = "extent"

# Read the table with all the landscape elements (le_table)
le_df = dbf.Dbf(wrkspc4+"\\"+le_table4)

# Read the table with all the population pairs (prs_table)
pairs = dbf.Dbf(wrkspc4+"\\"+prs_table4)

# Printing loop information
log = open(wrkspc4+"\\"+log_name4, 'a')
log.write("\n")
log.write("## Starting LCTA for straight lines\n")
log.close()

# Creates a tmp_tab
gp.CreateTable_management(wrkspc4, "temp_tab.dbf", "out_tab.dbf")

# create empty dictionaries to hold the names and coordinates by sample name
pointNameDic = {}
pointXYDic = {}

# get sample names and coordinates into dictionary
# place all the rows from the sample data points into a search cursor
cur = gp.SearchCursor(samp_locs4)
row = cur.Next()
i = 0
while row:
    name = str(row.GetValue(sampID4))
    pointNameDic[i] = name
    geom = row.Shape
    xy = geom.Centroid
    x_and_y = str(xy).split()
    pointXYDic[name] = x_and_y
    row = cur.Next()
    i = i + 1
del cur, row

# Create a directory that stores all the straight line paths
if not os.path.isdir(os.path.join(wrkspc4, 'str_line_paths')):
    os.mkdir(os.path.join(wrkspc4, 'str_line_paths'))

for pair in pairs:
    # Starting the time calculation for this loop
    start = time.time()

    # Setting start and end populations
    start_node = pair.__getitem__("NODE1")
    end_node = pair.__getitem__("NODE2")

    # Set the name of the straight-line path in this loop
    lcp_name = "sl_"+start_node.__str__()+"_"+end_node.__str__()+"_f_0"

    # If the straight line path already exists in the workdirectory, delete it
    if gp.Exists(lcp_name+".shp") == 1:
        gp.delete_management(lcp_name+".shp")

    # Check whether the straight line has already been calculated
    if gp.Exists('str_line_paths\\'+lcp_name+'.shp') == 1:
        gp.Copy_management('str_line_paths\\'+lcp_name+'.shp', lcp_name+'.shp')

    if gp.Exists(lcp_name+".shp") == 0:
        # Create a new feature class for the straight line path.
        gp.CreateFeatureclass(gp.workspace, lcp_name+".shp", "POLYLINE")

        # extract xy coordinates start_node
        x1 = float(pointXYDic[start_node.__str__()][0].replace(",", "."))
        y1 = float(pointXYDic[start_node.__str__()][1].replace(",", "."))

        # extract xy coordinates end_node
        x2 = float(pointXYDic[end_node.__str__()][0].replace(",", "."))
        y2 = float(pointXYDic[end_node.__str__()][1].replace(",", "."))

        # calculate the distance
        distance = math.sqrt(((x1 - x2) ** 2) + ((y1 - y2) ** 2))

        # Create lines in shapefile
        cur = gp.InsertCursor(lcp_name+".shp")
        array = gp.CreateObject("Array")
        pnt = gp.CreateObject("Point")

        pnt.X = x1
        pnt.Y = y1
        array.Add(pnt)
        pnt.X = x2
        pnt.Y = y2
        array.Add(pnt)

        row = cur.NewRow()
        row.Shape = array
        cur.InsertRow(row)

        del row, pnt, array, cur

        # Calculate the length of the straight-line path.
        gp.Addfield_management(lcp_name+".shp", "LENGTH", "DOUBLE")
        gp.CalculateField_management(lcp_name+".shp", "LENGTH", distance, "PYTHON")

        # Copy the straight line shapefile to the special folder.
        gp.Copy_management(lcp_name+".shp", 'str_line_paths\\'+lcp_name+".shp")

    # Extract the length and the cost of the straight-line path shapefile
    lc_database = dbf.Dbf(wrkspc4+"\\"+lcp_name+".dbf")
    length = ("%.2f" % sum([row["LENGTH"] for row in lc_database]))
    lc_database.close()

    # Buffers the straight-line path with a specific distance (buff).
    gp.Buffer_analysis(lcp_name+".shp", "buf_"+lcp_name+".shp", buff4, "FULL", "ROUND", "ALL")

    # Counts the number of pixels in the complete path.
    gp.ExtractByMask_sa("extent", "buf_"+lcp_name+".shp", "temp_rast")
    vat = gp.searchcursor("temp_rast")
    row_n = vat.next()
    all_pix_list = []
    while row_n:
        all_pix_list.append(row_n.count)
        row_n = vat.next()
    all_pix = sum(all_pix_list)
    del vat, row_n, all_pix_list
    gp.delete_management("temp_rast")

    # Fill the "temp_tab.dbf" for this loop
    temp_tab = dbf.Dbf(wrkspc4+"\\"+"temp_tab.dbf")
    temp_tab_row = temp_tab.newRecord()
    temp_tab_row["Field1"] = 0
    temp_tab_row["NODE1"] = start_node.__str__()
    temp_tab_row["NODE2"] = end_node.__str__()
    temp_tab_row["LENGTH"] = float(length)
    temp_tab_row["COST"] = float(0)
    temp_tab_row["ALL_PIX"] = float(all_pix)

    # Loop to add the proportion of each landscape element in the "temp_tab.dbf".
    for rec in le_df:
        le5 = rec.__getitem__("LANDEL")
        gp.ExtractByMask_sa(le5, "buf_"+lcp_name+".shp", "temp_rast")
        if gp.Exists("temp_rast") == 0:
            log = open(wrkspc4+"\\"+log_name4, 'a')
            log.write("!!!!!!!! ERROR: temp_rast not created\n")
            log.close()
        max_val = gp.GetRasterProperties_management("temp_rast", "MAXIMUM")
        if max_val > 0:
            vat = gp.searchcursor("temp_rast")
            row_n = vat.next()
            le_pix_list = []
            while row_n:
                le_pix_list.append(row_n.count)
                row_n = vat.next()
            le_pix = sum(le_pix_list)
            del vat, row_n, le_pix_list
        else:
            le_pix = 0
        temp_tab_row[le5] = float(le_pix)/float(all_pix)
        gp.delete_management("temp_rast")

    # Close the temp_tab.dbf
    temp_tab_row.store()
    temp_tab.close()

    # Delete the created variables
    del length, all_pix, le_pix, max_val, rec

    # Delete the straight-line path and transect shapefiles
    gp.delete_management(lcp_name+".shp")
    gp.delete_management("buf_"+lcp_name+".shp")

    # Print the results of this loop to the log file.
    end = time.time()
    log = open(wrkspc4+"\\"+log_name4, 'a')
    log.write("## Finished calculations between deme "+start_node.__str__()+" and "+end_node.__str__()+" in "+("%.1f" % (end - start))+" seconds \n")
    log.close()


# Rename the temp_tab.dbf
if gp.Exists("str_li_b_"+buff4.__str__()+".dbf") == 1:
    os.remove(os.path.join(wrkspc4, "str_li_b_"+buff4.__str__()+".dbf"))
os.rename(os.path.join(wrkspc4, "temp_tab.dbf"), os.path.join(wrkspc4, "str_li_b_"+buff4.__str__()+".dbf"))

# Close open data
le_df.close()
pairs.close()



